// const decode = require('urldecode')
const utils = require('./utils')

const handleSchemaRequest = (restRequest, controllers) => {
    const allowedMethods = ['get', 'post']
    if (allowedMethods.indexOf(restRequest.method) == -1){
        throw new Error('Invalid HTTP method: ' + restRequest.method)
        return
    }

    const resources = []
    const keys = Object.keys(controllers)
    keys.forEach(key => {
        const ctr = controllers[key]
        resources.push({
            name: key,
            collectionName: ctr.collectionName(),
            schema: ctr.schema()
        })
    })

    return resources
}

const handleCollectionRequest = (restRequest, controllers) => {
    return new Promise((resolve, reject) => {
        const allowedMethods = ['get', 'post']
        if (allowedMethods.indexOf(restRequest.method) == -1){
            reject(new Error('Invalid HTTP method: ' + restRequest.method))
            return
        }

        const ctr = controllers[restRequest.resource]
        if (ctr == null){
            reject(new Error('invalid resource: '+restRequest.resource))
            return
        }

        // check http method here:
        if (restRequest.method=='get'){
            const query = restRequest.event.queryStringParameters || {}
            ctr.get(query)
            .then(data => {
                resolve(data)
                return
            })
            .catch(err => {
                reject(err)
                return
            })
        }
        else if (restRequest.method=='post'){
            const body = utils.parseBody(restRequest.event)
            if (body == null){
                reject(new Error('Missing request body.'))
                return
            }

            ctr.post(body)
            .then(data => {
                resolve(data)
                return
            })
            .catch(err => {
                reject(err)
                return
            })
        }
        else {
            reject(new Error('invalid HTTP method: '+restRequest.method))
        }
    })
}

const handleRecordRequest = (restRequest, controllers) => {
    const allowedMethods = ['get', 'put', 'delete']
    return new Promise((resolve, reject) => {
        if (allowedMethods.indexOf(restRequest.method) == -1){
            reject(new Error('Invalid HTTP method: ' + restRequest.method))
            return
        }

        const ctr = controllers[restRequest.resource]
        if (ctr == null){
            reject(new Error('invalid resource: '+restRequest.resource))
            return
        }

        if (restRequest.method=='get'){
            ctr.getById(restRequest.id)
            .then(data => {
                resolve(data)
                return
            })
            .catch(err => {
                reject(err)
                return
            })
        }
        if (restRequest.method=='put'){
            const body = utils.parseBody(restRequest.event)
            if (body==null){
                reject(new Error('Missing request body'))
                return
            }

            ctr.put(restRequest.id, body)
            .then(data => {
                resolve(data)
                return
            })
            .catch(err => {
                reject(err)
                return
            })
        }
        if (restRequest.method=='delete'){
            const entityId = restRequest.id
            ctr.delete(entityId)
            .then(data => {
                resolve({id: entityId})
                return
            })
            .catch(err => {
                reject(err)
                return
            })
        }
    })
}

module.exports = (controllers, event, restRequest) => {
    return new Promise((resolve, reject) => {
        if (restRequest.type=='schema'){
            try {
                const resources = handleSchemaRequest(restRequest, controllers)
                resolve(resources)
                return
            }
            catch(err){
                reject(err)
                return
            }
            return
        }

        if (restRequest.type=='collection'){
            handleCollectionRequest(restRequest, controllers)
            .then(payload => {
                resolve(payload)
            })
            .catch(err => {
                reject(err)
            })

            return
        }

        if (restRequest.type=='record'){
            handleRecordRequest(restRequest, controllers)
            .then(payload => {
                resolve(payload)
            })
            .catch(err => {
                reject(err)
            })

            return
        }

        reject(new Error('Invalid Request'))
    })
}